/* Platform game 
 * Huge thanks to tile-based platform examples by Chris DeLeon
 * 
 * Written by M.Narvidas
 * 2012
 */
//
//outlinedText("It worked.",width/2, height/2-12); //used for debugging


// Images
PImage guy_stand, guy_run0, guy_run1, guy_run2, brick, gravity, enemy, gwall, fire, arrowN, arrowP, backg, arrowUPP, arrowUPN, spike;

//Settings
boolean android = true; //Set sketch to PC or Android mode. After implementing touches PC mode is not used anymore
boolean enemyFlip = true; //Flip the enemy sprite
int playerRadiusAOI = 20; //area of interest for colliding with enemy
int enemyRadiusAOI = 20; //area of interest for colliding with player
boolean healthReset = false;
boolean playerReset = false;

float checkX0, checkY0, checkX1, checkY1,checkTX1, checkTY1;


float cameraOffsetX; // camera offset to left or right
float startTime, currTime, hitTime;
float gravitation = 0.6;
int healthUpdate = 0;

//Objects
Player thePlayer = new Player();
Enemy theEnemy = new Enemy();
World theWorld = new World();
Health playerHealth = new Health(1, 0, 100, 100, 40, 60);

PFont font;

// Timer (not used in the game atm)
  int gameStartTimeSec, gameCurrentTimeSec;

// This is added to player's y velocity every frame
  float GRAVITY_POWER = gravitation; 

  int pointerCount;
  ArrayList points;   
  
  
  boolean hasrun = false;
  
  boolean leftPressed = false;
  boolean rightPressed = false;
  boolean upPressed = false;
  PVector arrowLeft,arrowRight, arrowUp;
  
  float arrowW;
  float arrowH;

  float lx;
  float ly;
  float rx;
  float ry;
  float ux;
  float uy;
    
void setup() { 
    
  arrowLeft = new PVector();
  arrowRight = new PVector();
  arrowUp = new PVector();

  arrowLeft.x = width/6;
  arrowLeft.y = height/1.3;
  arrowRight.x = arrowLeft.x+160;
  arrowRight.y = height/1.3;
  arrowUp.x = arrowLeft.x+530;
  arrowUp.y = height/1.3;
  
  if(android){orientation(LANDSCAPE);} // forces landscape on the sketch if android mode is on
  if (android == false) {size(800, 480); } // how large the window/screen is for the game (Android mode sets resolution automatically, PC mode does not)
  hitTime = 400; // 1000 millis = 1 second   // the counter
  startTime = millis();

  font = createFont("Arial-Black", 20);
  textFont(font);                                                                                                                                                                                                                                                                                                                                                   
  brick = loadImage("wall.png");
  guy_stand = loadImage("guy.png");
  guy_run0 = loadImage("guy1.png");  
  guy_run1 = loadImage("guy1.png");
  guy_run2 = loadImage("guy2.png");
  gravity = loadImage("gravity.png");
  enemy = loadImage("enemy.png");
  gwall = loadImage("grasswall.png");
  spike = loadImage("spikes.png");
  arrowN = loadImage("arrown.png");
  arrowUPP = loadImage("arrowupp.png");
  arrowUPN = loadImage("arrowupn.png");
  arrowP = loadImage("arrowp.png");
  backg = loadImage("background.png");
  
  arrowW = arrowN.width;
  arrowH = arrowN.height;
  
  lx=arrowLeft.x-arrowW;
  ly=arrowLeft.y;
  rx=arrowRight.x-160;
  ry=arrowRight.y;  
  ux=arrowUp.x;
  uy=arrowUp.y; 
  
  cameraOffsetX = 0.0;


  frameRate(24); // Set the Framerate
  resetGame(); // sets the instances

}

void resetGame() {
  
  thePlayer.reset(); 
  theWorld.reload(); 
  // reset timer in corner
  gameCurrentTimeSec = gameStartTimeSec = millis()/1000; // dividing by 1000 to turn milliseconds into seconds
}

Boolean gameWon() { 
  return false; //returned true if game was won (lets say defeated the final boss or whatever, don't know yet)
}

void outlinedText(String textWritten, float textX, float textY) {
  textFont(font); //load the font created before
  fill(0); // white for the upcoming text, drawn in each direction to make outline
  text(textWritten, textX-1, textY);
  text(textWritten, textX+1, textY);
  text(textWritten, textX, textY-1);
  text(textWritten, textX, textY+1);
  fill(255); // white for this next text, in the middle
  text(textWritten, textX, textY);
}

void updateCameraPosition() {
  int rightEdge = World.GRID_UNITS_WIDE*World.GRID_UNIT_SIZE-width; //Width of the world minus width of the screen, not to go over the limit of the world

  cameraOffsetX = thePlayer.position.x-width/2; //Center the view
  
  if (cameraOffsetX < 0) { //don't go over the limit
    cameraOffsetX = 0;
  }

  if (cameraOffsetX > rightEdge) { //don't go over the limit
    cameraOffsetX = rightEdge;
  }
}



void draw() {
  pushMatrix(); 
  translate(-cameraOffsetX, 0.0); // Affects all upcoming graphics calls, until popMatrix
  currTime = millis() - startTime;
  if ( currTime >= hitTime )
  {
    startTime = millis();
    // enemyFlip = !enemyFlip; // Now enemy uses collision istead of timer to flip
  }
  
  //call functions and methods to draw the game and etc.
  updateCameraPosition();
  theWorld.render(); 
  thePlayer.inputCheck();
  thePlayer.move();
  thePlayer.draw();
  theEnemy.checkEnemy();
  theEnemy.draw();

  popMatrix(); // Undoes transformations
  
  textAlign(LEFT); 
    smooth();
    outlinedText("Coins:"+thePlayer.coinsCollected +"/"+theWorld.coinsInStage, 35, height-400);
  
  //show the hp and check it  
  playerHealth.displayHP();
  playerHealth.driveHP();

 
    if (gameWon()) {
      textAlign(CENTER); // text alignment
      outlinedText("You won!\n", width/2, height/2-12);
    }
     drawArrows();
  }


boolean circleCircleIntersect(float cx1, float cy1, float cr1, float cx2, float cy2, float cr2) { //this function checks if two circles overlap. Used for enemy--player collision check
  if (dist(cx1, cy1, cx2, cy2) < cr1 + cr2) {
    return true;
  } 
  else {
    return false;
  }
}



