/* The world. (Level 1)
 * 
 */
 
  class World {
  int coinsInStage; // when we load a level, we count how many coins are in the level
  int coinRotateTimer; // number cycles, and is used to give coins a simple spinning effect
  
  
  static final int TILE_EMPTY = 0;
  static final int TILE_SOLID = 1;
  static final int TILE_COIN = 2;
  static final int TILE_SPIKE = 3;
  static final int TILE_START = 4; // the player starts where this is placed
  static final int TILE_CLOUDL = 5;
  static final int TILE_CLOUDR = 6;
  static final int TILE_SKY = 7;
  static final int TILE_FULLSKY = 8;
  static final int TILE_GRADIENTSKY = 9;
  static final int TILE_GRAVITY = 10;
  static final int TILE_ENEMY = 11;
  static final int TILE_GRASS= 12;
  
  static final int GRID_UNIT_SIZE = 48; // size, in pixels, of each world unit square
  // if the above number becomes too small, how the player's wall bumping is detected may need to be updated

  // these next 2 numbers need to match the dimensions of the example level grid below
  static final int GRID_UNITS_WIDE = 54;
  static final int GRID_UNITS_TALL = 10;

  int[][] levelOneTILES = new int[GRID_UNITS_TALL][GRID_UNITS_WIDE]; // the game checks this one during play

  // the game copies this into levelOneTILES each reset, returning coins that have since been cleared
  int[][] startingTILES = { 
  
    
{0, 0, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0},
{0, 0, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0},
{0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 2, 2, 2, 2, 10, 2, 2, 2, 2, 1, 0, 0, 0},
{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 2, 0, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 0, 0, 0},
{0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0},
{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 12, 12, 0, 0, 0, 0, 0, 0, 2, 2, 2, 2, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
{12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 12, 1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 2, 2, 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 2, 0, 0, 2, 0, 0, 1},
{1, 12, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 12, 12, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1},
{1, 1, 12, 12, 0, 0, 0, 11, 0, 2, 0, 12, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 1, 1},
{1, 1, 1, 1, 12, 12, 12, 12, 12, 12, 12, 12, 12, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 1, 1},





                       
                     };
                         
                         
                         

  World() { // Constructor
    coinRotateTimer = 0; // initialise 
  }

  // returns what type of tile is at a given pixel coordinate
  int worldSquareAt(PVector thisPosition) {
    
    float gridSpotX = thisPosition.x/GRID_UNIT_SIZE;
    float gridSpotY = thisPosition.y/GRID_UNIT_SIZE;

    //if goes out of the limits, treat as solids
    if (gridSpotX<0) {
      return TILE_SOLID;
    }
    if (gridSpotX>=GRID_UNITS_WIDE) {
      return TILE_SOLID;
    }
    if (gridSpotY<0) {
      return TILE_SOLID;
    }
    if (gridSpotY>=GRID_UNITS_TALL) {
      return TILE_SOLID;
    }

    return levelOneTILES[int(gridSpotY)][int(gridSpotX)];
  }

  // changes the tile at a given pixel coordinate to be a new tile type
  // currently used to replace TILE_COIN TILES with TILE_EMPTY TILES once collected
  void setSquareAtToThis(PVector thisPosition, int newTile) {
    int gridSpotX = int(thisPosition.x/GRID_UNIT_SIZE);
    int gridSpotY = int(thisPosition.y/GRID_UNIT_SIZE);

    if (gridSpotX<0 || gridSpotX>=GRID_UNITS_WIDE || 
      gridSpotY<0 || gridSpotY>=GRID_UNITS_TALL) {
      return; // can't change grid units outside the grid
    }

    levelOneTILES[gridSpotY][gridSpotX] = newTile;
  }

  // these helper functions help us correct for the player moving into a world tile
  float topOfSquare(PVector thisPosition) {
    int thisY = int(thisPosition.y);
    thisY /= GRID_UNIT_SIZE;
    return float(thisY*GRID_UNIT_SIZE);
  }
  float bottomOfSquare(PVector thisPosition) {
    if (thisPosition.y<0) {
      return 0;
    }
    return topOfSquare(thisPosition)+GRID_UNIT_SIZE;
  }
  float leftOfSquare(PVector thisPosition) {
    int thisX = int(thisPosition.x);
    thisX /= GRID_UNIT_SIZE;
    return float(thisX*GRID_UNIT_SIZE);
  }
  float rightOfSquare(PVector thisPosition) {
    if (thisPosition.x<0) {
      return 0;
    }
    return leftOfSquare(thisPosition)+GRID_UNIT_SIZE;
  }

  void reload() {
    coinsInStage = 0; // we count them while copying in level data

    for (int i=0;i<GRID_UNITS_WIDE;i++) {
      for (int ii=0;ii<GRID_UNITS_TALL;ii++) {
        
        if ( startingTILES[ii][i] == TILE_ENEMY ) {
          levelOneTILES[ii][i] = TILE_EMPTY;
          theEnemy.position.x = i*GRID_UNIT_SIZE+(GRID_UNIT_SIZE/2);
          theEnemy.position.y = ii*GRID_UNIT_SIZE+(GRID_UNIT_SIZE*45/46);
        } 
        
        
        if (startingTILES[ii][i] == TILE_START) { // player start position
          levelOneTILES[ii][i] = TILE_EMPTY; // put an empty tile in that spot

          // then update the player spot to the center of that tile
          thePlayer.position.x = i*GRID_UNIT_SIZE+(GRID_UNIT_SIZE/2);
          thePlayer.position.y = ii*GRID_UNIT_SIZE+(GRID_UNIT_SIZE/2);
        } 
        else {
          if (startingTILES[ii][i]==TILE_COIN) {
            coinsInStage++;
          }
          levelOneTILES[ii][i] = startingTILES[ii][i];
        }
      }
    }
  }
  

  void render() { // draw the world
  
   int sizeB =  ((GRID_UNIT_SIZE * GRID_UNITS_WIDE) / backg.width)+1; //This divides the total width by the background width to get the number of repetitions for drawing the background. 1 is added because of the nature of division in programming
   int pusher = 0; // used to repeat the same background over and over
   for (int i=0; i<sizeB; i++)  { //repeat till 
   image(backg,pusher,0,backg.width,backg.height);
   pusher = pusher + backg.width;  //start drawing the next background exactly where previous one finished
   }
  
    // Gives the "spinning" effect to the coins in stage
    coinRotateTimer--;
    if (coinRotateTimer<-GRID_UNIT_SIZE/3) {
      coinRotateTimer = GRID_UNIT_SIZE/3;
    }

    for (int i=0;i<GRID_UNITS_WIDE;i++) { // for each column
      for (int ii=0;ii<GRID_UNITS_TALL;ii++) { // for each tile in that column

        if (levelOneTILES[ii][i]==TILE_COIN) { // if it's a coin, draw the coin
          stroke(0); // black outline
          fill(250, 250, 50); // yellow colour in RGB
          ellipse(i*GRID_UNIT_SIZE+(GRID_UNIT_SIZE/2), ii*GRID_UNIT_SIZE+(GRID_UNIT_SIZE/2), // center of grid spot
          abs(coinRotateTimer), GRID_UNIT_SIZE/2); // spin size wide, 1/2 height of box tall
        }
        
        if (levelOneTILES[ii][i]==TILE_SPIKE) { 
        image(spike, i*GRID_UNIT_SIZE, ii*GRID_UNIT_SIZE, GRID_UNIT_SIZE-1, GRID_UNIT_SIZE);
        }
        if (levelOneTILES[ii][i]==TILE_SOLID) { 
        image(brick, i*GRID_UNIT_SIZE, ii*GRID_UNIT_SIZE, GRID_UNIT_SIZE-1, GRID_UNIT_SIZE);
        }
        if(levelOneTILES[ii][i]==TILE_GRAVITY) { 
        image(gravity,i*GRID_UNIT_SIZE,ii*GRID_UNIT_SIZE, GRID_UNIT_SIZE,GRID_UNIT_SIZE-1);
        }
        if(levelOneTILES[ii][i]==TILE_GRASS) { 
        image(gwall,i*GRID_UNIT_SIZE, ii*GRID_UNIT_SIZE, GRID_UNIT_SIZE-1, GRID_UNIT_SIZE);
        }
      }
    }
  }
}

